@@ -52,12 +52,25 @@ import org.kohsuke.stapler.export.Exported;
      */
     @ExportedBean
     public static final class DiskSpace extends OfflineCause implements Serializable {
+        private final String path;
         @Exported
         public final long size;
         
         private boolean triggered;
 
+        /**
+         * @deprecated as of 1.467
+         */
         public DiskSpace(long size) {
+            this(".",size);
+        }
+
+        /**
+         * @param path
+         *      Specify the file path that was monitored.
+         */
+        public DiskSpace(String path, long size) {
+            this.path = path;
             this.size = size;
         }
 
@@ -65,6 +78,14 @@ import org.kohsuke.stapler.export.Exported;
         public String toString() {
             return String.valueOf(size);
         }
+        
+        /**
+         * The path that was checked
+         */
+        @Exported
+        public String getPath() {
+            return path;
+        }
 
         /**
          * Gets GB left.
@@ -125,7 +146,7 @@ import org.kohsuke.stapler.export.Exported;
                 }
             }
 
-            return new DiskSpace((long)(Double.parseDouble(size.trim())*multiplier));
+            return new DiskSpace("", (long)(Double.parseDouble(size.trim())*multiplier));
         }
 
         private static final long serialVersionUID = 2L;
@@ -146,7 +167,7 @@ import org.kohsuke.stapler.export.Exported;
             try {
                 long s = f.getUsableSpace();
                 if(s<=0)    return null;
-                return new DiskSpace(s);
+                return new DiskSpace(f.getCanonicalPath(), s);
             } catch (LinkageError e) {
                 // pre-mustang
                 return null;
