@@ -23,6 +23,7 @@
  */
 package hudson.util;
 
+import com.google.common.collect.ImmutableList;
 import com.google.common.collect.ImmutableMap;
 import com.thoughtworks.xstream.XStream;
 import com.thoughtworks.xstream.mapper.AnnotationMapper;
@@ -41,11 +42,15 @@ import com.thoughtworks.xstream.io.HierarchicalStreamReader;
 import com.thoughtworks.xstream.io.HierarchicalStreamWriter;
 import com.thoughtworks.xstream.mapper.CannotResolveClassException;
 import hudson.diagnosis.OldDataMonitor;
+import hudson.util.xstream.ImmutableSetConverter;
+import hudson.util.xstream.ImmutableSortedSetConverter;
 import jenkins.model.Jenkins;
 import hudson.model.Label;
 import hudson.model.Result;
 import hudson.model.Saveable;
+import hudson.util.xstream.ImmutableListConverter;
 import hudson.util.xstream.ImmutableMapConverter;
+import hudson.util.xstream.MapperDelegate;
 
 import java.lang.reflect.Constructor;
 import java.lang.reflect.InvocationTargetException;
@@ -62,6 +67,11 @@ public class XStream2 extends XStream {
 
     private final Map<String,Class<?>> compatibilityAliases = new ConcurrentHashMap<String, Class<?>>();
 
+    /**
+     * Hook to insert {@link Mapper}s after they are created.
+     */
+    private MapperInjectionPoint mapperInjectionPoint;
+
     public XStream2() {
         init();
     }
@@ -101,6 +111,9 @@ public class XStream2 extends XStream {
 
         registerConverter(new RobustCollectionConverter(getMapper(),getReflectionProvider()),10);
         registerConverter(new ImmutableMapConverter(getMapper(),getReflectionProvider()),10);
+        registerConverter(new ImmutableSortedSetConverter(getMapper(),getReflectionProvider()),10);
+        registerConverter(new ImmutableSetConverter(getMapper(),getReflectionProvider()),10);
+        registerConverter(new ImmutableListConverter(getMapper(),getReflectionProvider()),10);
         registerConverter(new ConcurrentHashMapConverter(getMapper(),getReflectionProvider()),10);
         registerConverter(new CopyOnWriteMap.Tree.ConverterImpl(getMapper()),10); // needs to override MapConverter
         registerConverter(new DescribableList.ConverterImpl(getMapper()),10); // explicitly added to handle subtypes
@@ -108,7 +121,7 @@ public class XStream2 extends XStream {
 
         // this should come after all the XStream's default simpler converters,
         // but before reflection-based one kicks in.
-        registerConverter(new AssociatedConverterImpl(this),-10);
+        registerConverter(new AssociatedConverterImpl(this), -10);
     }
 
     @Override
@@ -118,13 +131,50 @@ public class XStream2 extends XStream {
             public String serializedClass(Class type) {
                 if (type != null && ImmutableMap.class.isAssignableFrom(type))
                     return super.serializedClass(ImmutableMap.class);
+                else if (type != null && ImmutableList.class.isAssignableFrom(type))
+                    return super.serializedClass(ImmutableList.class);
                 else
                     return super.serializedClass(type);
             }
         });
         AnnotationMapper a = new AnnotationMapper(m, getConverterRegistry(), getClassLoader(), getReflectionProvider(), getJvm());
         a.autodetectAnnotations(true);
-        return a;
+
+        mapperInjectionPoint = new MapperInjectionPoint(a);
+
+        return mapperInjectionPoint;
+    }
+
+    public Mapper getMapperInjectionPoint() {
+        return mapperInjectionPoint.getDelegate();
+    }
+
+    /**
+     * This method allows one to insert additional mappers after {@link XStream2} was created,
+     * but because of the way XStream works internally, this needs to be done carefully.
+     * Namely,
+     *
+     * <ol>
+     * <li>You need to {@link #getMapperInjectionPoint()} wrap it, then put that back into {@link #setMapper(Mapper)}.
+     * <li>The whole sequence needs to be synchronized against this object to avoid a concurrency issue.
+     * </ol>
+     */
+    public void setMapper(Mapper m) {
+        mapperInjectionPoint.setDelegate(m);
+    }
+
+    final class MapperInjectionPoint extends MapperDelegate {
+        public MapperInjectionPoint(Mapper wrapped) {
+            super(wrapped);
+        }
+
+        public Mapper getDelegate() {
+            return delegate;
+        }
+
+        public void setDelegate(Mapper m) {
+            delegate = m;
+        }
     }
 
     /**
