@@ -31,7 +31,7 @@ typedef struct _SECURITY_ATTRIBUTES {
 } SECURITY_ATTRIBUTES,
  *PSECURITY_ATTRIBUTES,
  *LPSECURITY_ATTRIBUTES;*/
-  public static class SECURITY_ATTRIBUTES extends Structure {
+  class SECURITY_ATTRIBUTES extends Structure {
     public int nLength;
     public Pointer lpSecurityDescriptor;
     public boolean bInheritHandle;
@@ -42,7 +42,7 @@ typedef struct _FILETIME {
     DWORD dwLowDateTime;
     DWORD dwHighDateTime;
 } FILETIME, *PFILETIME, *LPFILETIME;*/
-  public static class FILETIME extends Structure {
+  class FILETIME extends Structure {
     public int dwLowDateTime;
     public int dwHighDateTime;
   }
